#include "mbed.h"
#include <cstdint>
#include <random>
#include <iostream>

#define DET_T 0.01 //10 ms (une periode fondamentale des tickers)
#define NB_OBJET 20 // nombre d'objets à traiter, longueur de la liste en cours de traitement

// 1er ticker déclenchement servo
// Nombre de période avant déclenchement pour le moteur 1, 2, 3
#define N_1  230
#define N_2  520
#define N_3  790

// 2eme ticker retour servo
// Nombre de période avant retour pour les moteurs 1, 2, 3
#define N_retour 140

Ticker toggle_ticker;

//Ticker 1
// On établit une liste de longueur finie qui contient les états détectés en cours et leur temps qui décroit de N à 0.
int objet_liste[NB_OBJET]={0}; // [1, 2, 0, 0, 1, 3...] valeur pour pousser moteur 1 moteur 2 ou 3. La valeur 0 indique, aucune affectation
int n_liste[NB_OBJET]={0}; // [300, 500, 0, 0, 500...] : liste des compteurs de temps qui décroissent à 0. 
// contient les temps N associés aux états occupés détecté. A la détection la valeur est N puis décroit toutes les 10ms jusqu'à 0 où l'état devient disponible
// A 0 l'état est libre


//Ticker 2
// Retour des moteurs à leur position initiale
int retour_liste[NB_OBJET]={0};
int n_retour_liste[NB_OBJET]={0}; // [300, 500, 0, 0, 500...] : liste des compteurs de temps qui décroissent à 0. 

int delay = 0; // initialisation du delai minimal entre 2 détéctions d'objet (100ms)

//VARIABLES RECEPTION PC NUCLEO 
int objet_detecte = 0;
int run = 0;
int initialisation = 0;

//MOTEUR
// Alimentation en 5,7V continu sur le L298, un courant de 0,7-0,8A est créé
PwmOut clock_pwm(A1);
DigitalOut reset(A2);
DigitalOut half(A3);
DigitalOut sens_clockwise(A4);
DigitalOut enable(A5);

DigitalIn detecteur(D13);


//Test envoi données leds
DigitalOut led(D6);
DigitalOut led_2(D5);
DigitalOut led_3(D4);

// Connection au programme python pour l'envoie et la reception de données
UnbufferedSerial      my_pc(USBTX, USBRX);
char data;
void reception_pc_nucleo(void);


//SERVO 1
// Delivre 3,3V en PWM
PwmOut servo_mot1(D8);

//SERVO 2
// Delivre 3,3V en PWM
PwmOut servo_mot2(D9);

//SERVO 3
// Delivre 3,3V en PWM
PwmOut servo_mot3(D10);

int led_activated;

// 1er ticker déclenchement servo
void init_compteur(int etat_libre) // initialise un compteur quand un objet est detecté
{
     
    if(objet_liste[etat_libre]==1) // si l'objet detecté doit être poussé par le servo 1
    {
        n_liste[etat_libre] = N_1;
    }

    if(objet_liste[etat_libre]==2) // si l'objet detecté doit être poussé par le servo 2
    {
        n_liste[etat_libre] = N_2;
    }

    if(objet_liste[etat_libre]==3) // si l'objet detecté doit être poussé par le servo 3
    { 
        n_liste[etat_libre] = N_3;
    }

}

// 2eme ticker retour
void init_compteur_retour(int i) // initialise un compteur quand le servo commence à pousser pour qu'il puisse revenir par la suite
{
    n_retour_liste[i] = N_retour;
}

void decrementation()  //décrémente, le compteur de N à 0
{   
    if(run == 1)
    {
        int i = 0;
        for(i=0; i<NB_OBJET; i++)
        {
            if(n_liste[i] != 0) // && objet_liste[i]!=0 pas besoin car pas besoin de cette vérif car la décrementation arrive à 0 automatiquement pas nég
            {
                n_liste[i]--;
            }
            if(n_retour_liste[i]!=0) // && retour_liste[i]!=0 pas besoin car pas besoin de cette vérif car la décrementation arrive à 0 automatiquement pas nég
            {
                n_retour_liste[i]--;
            }  
        }
        // delai de détection au passage du cube, = temps de passage pour ne pas continuellement détecter quand le cube est devant
        if(delay != 0)
            delay --;
    }
}

int check_etat_libre() // regarde continuement la case la plus proche dans la liste qui est vide pour y associer le prochain objet
{
    int i=0; // indice de check d'un état libre
    while(objet_liste[i]!=0)
    {
        i++;
    }
    return i;
}

int main() 
{
    //MOTEUR PAS à PAS
    clock_pwm.period_ms(1);
    clock_pwm.write(0.5);

    reset=1; //Initialisation obligatoire
    half=1; 
    sens_clockwise=1;//1 sens horaire 0 sens anti-horaire
    enable = 0;
    int a = 0;

    //SERVOMOTEURS
    int i, time = 1500;
    servo_mot1.period_ms(10);         // Initialisation période
    servo_mot2.period_ms(10);         // Initialisation période
    servo_mot3.period_ms(10);         // Initialisation période

    //CONNECTION PYTHON
    my_pc.baud(115200);
    my_pc.attach(&reception_pc_nucleo, UnbufferedSerial::RxIrq);

    // initialisation du ticker pour la décrementation
    toggle_ticker.attach(&decrementation, DET_T);

    while(1)
    {
        if(run == 1)
        {
            enable=1;
            led_2 = 1;
            led = 0;
        }
        if(run == 0)
        {
            enable=0; // Mise en marche enable et reset obligatoire
            led_2 = 0;
            led = 1;
        }

        if(detecteur == 1 && delay==0) // si le capteur a detecté un objet et si le délai de detection est à 
        {
            char message = 'd';
            my_pc.write(&message,1);// on envoie l'info au pc comme quoi on a eu une detection donc il faut prendre une photo ----> A FAIRE
            delay = 100;// réduit ma frame
        }

        if(objet_detecte != 0) // si le pc a envoyé l'info qu'un objet a été detecté
        {
            int etat_libre = check_etat_libre();  // On a trouvé une case libre i dans n_liste : aucun objet en cours
            objet_liste[etat_libre]=objet_detecte;
            
            init_compteur(etat_libre);
            objet_detecte = 0; // on remet l'état à 0 pour ne passer qu'une fois dans la condition if
        }
        
        for(i=0 ; i<NB_OBJET; i++)
        {
            if(n_liste[i]==0 && objet_liste[i]!=0)
            {

                if(objet_liste[i] == 1)
                {
                    servo_mot1.pulsewidth_us(2400);
                    init_compteur_retour(i);
                    objet_liste[i] = 0;
                    retour_liste[i] = 1;
                }

                if(objet_liste[i] == 2)
                {
                    servo_mot2.pulsewidth_us(2400);
                    init_compteur_retour(i);
                    objet_liste[i] = 0;
                    retour_liste[i] = 2;
                }
                
                if(objet_liste[i] == 3)
                {
                    servo_mot3.pulsewidth_us(2400);
                    init_compteur_retour(i);
                    objet_liste[i] = 0;
                    retour_liste[i] = 3;

                }
            }
        }

        for(i=0 ; i<NB_OBJET; i++)
        {
            if(n_retour_liste[i]==0 && retour_liste[i]!=0)
            {
                if(retour_liste[i] == 1)
                {
                    servo_mot1.pulsewidth_us(1000);
                    retour_liste[i] = 0;
                }
                
                if(retour_liste[i] == 2)
                {
                    servo_mot2.pulsewidth_us(1000);
                    retour_liste[i] = 0;
                }
                
                if(retour_liste[i] == 3)
                {
                    servo_mot3.pulsewidth_us(1000);
                    retour_liste[i] = 0;
                }
            }
        }

    }
}

void reception_pc_nucleo(void)
{
    my_pc.read(&data, 1);     // get the received byte
    
    // Information sur le servomoteur qu'il faut "activer" pour un objet
    if(data == '1')
    {
        objet_detecte = 1;
    }
    if(data == '2')
    {    
        objet_detecte = 2; 
    }
    if(data == '3')
    {    
        objet_detecte = 3; 
    }

    // Information de lancement global du convoyeur
    if(data == 'r')
    {    
        run = 1; 
    }

    // Information d'arret/relance global du convoyeur
    if(data == 's')
    {    
        run = 0; 
    }

    // Information d'initialisation globale du convoyeur 
    // if(data == 'init')
    // {    
    //     initialisation = 1;
    //     run = 0; 
    // }

    // my_pc.write(&data, 1);    // echo of the byte received
}


